﻿using adminBlazor.Factories;
using adminBlazor.Models;
using System.Transactions;

namespace TestUnit
{
    public class TranslationFactory_UT
    {
        public static IEnumerable<object[]> Dataset()
        {
            yield return new object[]
            {
                new TranslationModel[]
                {
                    new()
                    {
                        Id = 1,
                        FirstWord = "firstword",
                        SecondWord = "secondword"
                    },
                    new()
                    {
                        Id = 2,
                        FirstWord = "firstword2",
                        SecondWord = "secondword2"
                    },
                    new()
                    {
                        Id = 3,
                        FirstWord = "firstword3",
                        SecondWord = "secondword3"
                    }
                }
            };
        }
        [Theory]
        [MemberData(nameof(Dataset))]
        public void Create_Validation(TranslationModel[] translationMdls)
        {
            foreach (var translationMdl in translationMdls)
            {
                var translation = TranslationFactory.Create(translationMdl);

                Assert.Equal(translationMdl.Id, translation.Id);
                Assert.Equal(translationMdl.FirstWord, translation.FirstWord);
                Assert.Equal(translationMdl.SecondWord, translation.SecondWord);
            }
        }

        [Theory]
        [MemberData(nameof(Dataset))]
        public void Update_Validation(TranslationModel[] translationMdls)
        {
            foreach (var translationMdl in translationMdls)
            {
                var translation = TranslationFactory.Create(translationMdl);

                translationMdl.FirstWord = "a";
                translationMdl.SecondWord = "b";

                TranslationFactory.Update(translation, translationMdl);

                Assert.Equal(translationMdl.FirstWord, translation.FirstWord);
                Assert.Equal(translationMdl.SecondWord, translation.SecondWord);
            }
        }

        [Theory]
        [MemberData(nameof(Dataset))]
        public void ToModel_Validation(TranslationModel[] translationMdls)
        {
            foreach (var translationMdl in translationMdls)
            {
                var translation = TranslationFactory.Create(translationMdl);
                var translationMdl2 = TranslationFactory.ToModel(translation);

                Assert.Equal(translationMdl.Id, translationMdl2.Id);
                Assert.Equal(translationMdl.FirstWord, translationMdl2.FirstWord);
                Assert.Equal(translationMdl.SecondWord, translationMdl2.SecondWord);
            }
        }
    }
}