using adminBlazor.Factories;
using adminBlazor.Models;
using adminBlazor.Pages;

namespace TestUnit
{
    public class VocListFactory_UT
    {
        public static IEnumerable<object[]> Dataset()
        {
            yield return new object[]
            {
                new VocabularyListModel[]
                {
                    new ()
                    {
                        Id = 0,
                        Name = "name",
                        Image = new byte[] { 0x00 },
                        Aut = 42,
                        Translations = new List<TranslationModel>()
                        {
                            new()
                            {
                                Id = 1,
                                FirstWord = "firstword",
                                SecondWord = "secondword"
                            }
                        }
                    },
                    new()
                    {
                        Id = 1,
                        Name = "name2",
                        Aut = 50,
                        Translations = new List<TranslationModel>()
                        {
                            new()
                            {
                                Id = 2,
                                FirstWord = "firstword2",
                                SecondWord = "secondword2"
                            }
                        }
                    },
                    new()
                    {
                        Id = 1,
                        Name = "name3",
                        Image = new byte[] { 0x00, 0x01, 0x02 },
                        Aut = 60,
                        Translations = new List<TranslationModel>()
                        {
                            new()
                            {
                                Id = 3,
                                FirstWord = "firstword3",
                                SecondWord = "secondword3"
                            }
                        }
                    }
                }                
            };
        }
        [Theory]
        [MemberData(nameof(Dataset))]
        public void Create_Validation(VocabularyListModel[] vocabListMdls)
        {
            foreach (var vocabListMdl in vocabListMdls)
            {
                if (vocabListMdl.Image != null)
                {
                    vocabListMdl.ImageBase64 = Convert.ToBase64String(vocabListMdl.Image);
                }
                var vocabList = VocListFactory.Create(vocabListMdl);

                Assert.Equal(vocabListMdl.Id, vocabList.Id);
                Assert.Equal(vocabListMdl.Name, vocabList.Name);
                Assert.Equal(vocabListMdl.Aut, vocabList.Aut);
                Assert.Equal(vocabListMdl.Image, vocabList.Image);
                if (vocabList.Translations != null)
                {
                    Assert.Single(vocabList.Translations);
                }
                if (vocabListMdl.Image != null)
                {
                    Assert.Equal(vocabListMdl.ImageBase64, vocabList.ImageBase64);
                }
                else
                {
                    Assert.Null(vocabList.ImageBase64);
                }
            }
        }

        [Theory]
        [MemberData(nameof(Dataset))]
        public void Update_Validation(VocabularyListModel[] vocabListMdls)
        {
            foreach (var vocabListMdl in vocabListMdls)
            {
                if (vocabListMdl.Image != null)
                {
                    vocabListMdl.ImageBase64 = Convert.ToBase64String(vocabListMdl.Image);
                }
                var vocabList = VocListFactory.Create(vocabListMdl);

                vocabListMdl.Name = "a";
                vocabListMdl.Aut = 100;

                VocListFactory.Update(vocabList, vocabListMdl);

                Assert.Equal(vocabListMdl.Id, vocabList.Id);
                Assert.Equal(vocabListMdl.Name, vocabList.Name);
                Assert.Equal(vocabListMdl.Aut, vocabList.Aut);
                if (vocabList.Translations != null)
                {
                    Assert.Single(vocabList.Translations);
                }
                if (vocabListMdl.Image != null)
                {
                    Assert.Equal(vocabListMdl.ImageBase64, vocabList.ImageBase64);
                }
            }
        }

        [Theory]
        [MemberData(nameof(Dataset))]
        public void ToModel_Validation(VocabularyListModel[] vocabListMdls)
        {
            foreach (var vocabListMdl in vocabListMdls)
            {
                if (vocabListMdl.Image != null)
                {
                    vocabListMdl.ImageBase64 = Convert.ToBase64String(vocabListMdl.Image);
                }
                var vocabList = VocListFactory.Create(vocabListMdl);
                var vocabListMdl2 = VocListFactory.ToModel(vocabList, Array.Empty<byte>());

                Assert.Equal(vocabListMdl.Id, vocabListMdl2.Id);
                Assert.Equal(vocabListMdl.Name, vocabListMdl2.Name);
                Assert.Equal(vocabListMdl.Aut, vocabListMdl2.Aut);
                Assert.Equal(vocabListMdl.Image, vocabListMdl2.Image);
                if (vocabListMdl2.Translations != null)
                {
                    Assert.Single(vocabListMdl2.Translations);
                }
                if (vocabList.ImageBase64 != null)
                {
                    Assert.Equal(vocabList.ImageBase64, vocabListMdl2.ImageBase64);
                }
                else
                {
                    Assert.Equal("", vocabListMdl2.ImageBase64);
                }
            }
        }
    }
}